@@ -17,6 +17,7 @@ import time
 import types
 import scipy.special
 _pausemode=True
+import string
 
 class SASDict(object):
     """Small Angle Scattering results in a dictionary-like representation.
@@ -43,11 +44,13 @@ class SASDict(object):
     data.plot('.-',linewidth=3,markersize=5,label='test plot')
     
     """
-    _q=None
-    _Intensity=None
-    _Error=None
-    _Area=None
+    qtolerance=0.001
+    __instances=0
     def __init__(self,**kwargs):
+        self._q=None
+        self._Intensity=None
+        self._Error=None
+        self._Area=None
         try:
             self._q=np.array(kwargs['q'])
         except KeyError:
@@ -56,20 +59,27 @@ class SASDict(object):
             if len(kwargs['Intensity'])!=len(self._q):
                 raise ValueError('Length of Intensity should be equal to length of q.')
             self._Intensity=np.array(kwargs['Intensity'])
+        except TypeError:
+            self._Intensity=None
         except KeyError:
             raise ValueError('SASDict has to be initialized with Intensity given.')
         try:
             if len(kwargs['Error'])!=len(self._q):
                 raise ValueError('Length of Error should be equal to length of q.')
             self._Error=np.array(kwargs['Error'])
+        except TypeError:
+            self._Error=None
         except KeyError:
             self._Error=np.zeros(self._Intensity.shape)
         try:
             if len(kwargs['Area'])!=len(self._q):
-                raise ValueError('Length of Error should be equal to length of q.')
+                raise ValueError('Length of Area should be equal to length of q.')
             self._Area=np.array(kwargs['Area'])
+        except TypeError:
+            self._Area=None
         except KeyError:
             self._Area=np.zeros(self._Intensity.shape)
+        SASDict.__instances+=1
     def get_q(self):
         return self._q
     def get_Intensity(self):
@@ -90,7 +100,7 @@ class SASDict(object):
             self._Error=None
             self._Area=None
     def set_s(self,s1):
-        return self.set_q(self,s1/(2*np.pi))
+        return self.set_q(s1/(2*np.pi))
     def set_Intensity(self,I1):
         I1=np.array(I1)
         if (len(self._q)==len(I1)):
@@ -98,7 +108,7 @@ class SASDict(object):
         else:
             raise ValueError("Intensity should be of the same length as q.")
     def set_Error(self,E1):
-        E1=np.array(I1)
+        E1=np.array(E1)
         if (len(self._q)==len(E1)):
             self._Error=E1
         else:
@@ -115,7 +125,10 @@ class SASDict(object):
     Error=property(fget=get_Error,fset=set_Error,doc='Absolute error of scattered intensity')
     Area=property(fget=get_Area,fset=set_Area,doc='Effective area during integration')
     def save(self,filename):
-        return B1io.write1dsasdict(self,filename)
+        f=open(filename,'wt')
+        f.write('#%s\n'%string.join([str(k) for k in self.keys()]))
+        np.savetxt(f,np.array(self))
+        f.close()
     def keys(self):
         list=[]
         if self._q is not None:
@@ -181,10 +194,27 @@ class SASDict(object):
         pylab.plot(self._q,self._Intensity,*args,**kwargs)
     def errorbar(self,*args,**kwargs):
         pylab.errorbar(self._q,self._Intensity,self._Error,*args,**kwargs)
+    def _check_compat(self,x,die=False):
+        if len(x._q)!=len(self._q):
+            if die:
+                raise ValueError('Incompatible SAS dicts (q-scales have different lengths)!')
+            return False
+        if (2*np.absolute(x._q-self._q)/(x._q+self._q)).sum()/len(self._q)>SASDict.qtolerance:
+            if die:
+                raise ValueError('Incompatible SAS dicts (q-scales are different)!')
+            return False
+        return True
     def __imul__(self,x):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            self._check_compat(x,die=True)
+            self._q=0.5*(self._q+x._q)
+            self._Intensity=self._Intensity*x._Intensity
+            self._Error=np.sqrt((x._Intensity*self._Error)**2+(self._Intensity*x._Error)**2)
+            self._Area=None
+            return self
         else:
             val=x
             err=0
@@ -195,6 +225,9 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            self*=1/x
+            return self
         else:
             val=x
             err=0
@@ -205,6 +238,13 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            self._check_compat(x,die=True)
+            self._q=0.5*(self._q+x._q)
+            self._Intensity=self._Intensity+x._Intensity
+            self._Error=np.sqrt(self._Error**2+x._Error**2)
+            self._Area=None
+            return self
         else:
             val=x
             err=0
@@ -215,6 +255,13 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            self._check_compat(x,die=True)
+            self._q=0.5*(self._q+x._q)
+            self._Intensity=self._Intensity-x._Intensity
+            self._Error=np.sqrt(self._Error**2+x._Error**2)
+            self._Area=None
+            return self
         else:
             val=x
             err=0
@@ -225,6 +272,13 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            self._check_compat(x,die=True)
+            q1=0.5*(self._q+x._q)
+            Intensity1=self._Intensity*x._Intensity
+            Error1=np.sqrt((x._Intensity*self._Error)**2+(self._Intensity*x._Error)**2)
+            Area1=None
+            return SASDict(q=q1,Intensity=Intensity1,Error=Error1,Area=Area1)
         else:
             val=x
             err=0
@@ -235,6 +289,8 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            return self*(1/x)
         else:
             val=x
             err=0
@@ -246,6 +302,9 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            self._check_compat(x,die=True)
+            return SASDict(q=0.5*(self._q+x._q),Intensity=self._Intensity+x._Intensity,Error=np.sqrt(self._Error**2+x._Error**2),Area=None)
         else:
             val=x
             err=0
@@ -256,6 +315,9 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            self._check_compat(x,die=True)
+            return SASDict(q=0.5*(self._q+x._q),Intensity=self._Intensity-x._Intensity,Error=np.sqrt(self._Error**2+x._Error**2),Area=None)
         else:
             val=x
             err=0
@@ -271,6 +333,8 @@ class SASDict(object):
         if isinstance(x,tuple) and len(x)==2:
             val=x[0]
             err=x[1]
+        elif isinstance(x,SASDict):
+            return x*(1/self)
         else:
             val=x
             err=0
@@ -287,8 +351,8 @@ class SASDict(object):
         if exponent==0:
             return SASDict(q=self._q,Intensity=np.zeros(self._q.shape),Error=np.zeros(self._q.shape),Area=self._Area)
         else:
-            return SASDict(q=self._q,Intensity=np.pow(self._Intensity,exponent),
-                           Error=self._Error*np.absolute((exponent)*np.pow(self._Intensity,exponent-1)))
+            return SASDict(q=self._q,Intensity=np.power(self._Intensity,exponent),
+                           Error=self._Error*np.absolute((exponent)*np.power(self._Intensity,exponent-1)))
     def __array__(self):
         a=np.array(zip(*(self.values())),dtype=zip(self.keys(),[np.double]*len(self.keys())))
         return a
@@ -306,9 +370,9 @@ class SASDict(object):
         return self
     def sanitize(self,accordingto='Intensity',thresholdmin=0,thresholdmax=np.inf,function=None):
         if hasattr(function,'__call__'):
-            indices=function(self.getattr(accordingto))
+            indices=function(self.__getattribute__(accordingto))
         else:
-            indices=(self.getattr(accordingto)>thresholdmin) & (self.getattr(accordingto)<thresholdmax)
+            indices=(self.__getattribute__(accordingto)>thresholdmin) & (self.__getattribute__(accordingto)<thresholdmax)
         if self._q is not None:
             self._q=self._q[indices]
         if self._Intensity is not None:
@@ -323,13 +387,40 @@ class SASDict(object):
         y=self._Intensity*self._q**exponent
         err=self._Error*self._q**exponent
         m=np.trapz(y,x)
-        dm=errtapz(x,err)
+        dm=errtrapz(x,err)
         if errorrequested:
            return (m,dm) 
         else:
             return m
     def integral(self,errorrequested=False):
         return self.modulus(errorrequested=errorrequested)
+    def __len__(self):
+        return len(self.keys())
+    def __del__(self):
+        if self._q is not None:
+            del self._q
+        if self._Intensity is not None:
+            del self._Intensity
+        if self._Error is not None:
+            del self._Error
+        if self._Area is not None:
+            del self._Area
+        SASDict.__instances-=1
+#        print "An instance of SASDict has been disposed of. Remaining instances:",SASDict.__instances
+
+class SASImage(object):
+    def __init__(self,A,Aerr=None,param=None,mask=None):
+        self._A=A
+        self._Aerr=Aerr
+        self._param=param
+        self._mask=mask
+    def __getitem__(self,item):
+        try:
+            return self._param[item]
+        except KeyError:
+            raise
+    
+
 def trimq(data,qmin=-np.inf,qmax=np.inf):
     """Trim the 1D scattering data to a given q-range
     
@@ -767,7 +858,7 @@ def inv_error(A,DA):
     Output:
         The error of the inverse matrix
     """
-    B=np.inv(A);
+    B=np.linalg.linalg.inv(A);
     return np.sqrt(np.dot(np.dot(B**2,DA**2),B**2))
     
 def multiple(list,comparefun=(lambda a,b:(a==b))):
